@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 #
 # This file is part of Glances.
 #
-# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2024 Nicolas Hennion <nicolas@nicolargo.com>
 #
 # SPDX-License-Identifier: LGPL-3.0-only
 #
@@ -12,16 +11,24 @@
 import argparse
 import sys
 import tempfile
+from logging import DEBUG
+from warnings import simplefilter
 
-from glances import __version__, psutil_version
-from glances.compat import input, disable, enable, PY3
+try:
+    import shtab
+except ImportError:
+    shtab_tag = False
+else:
+    shtab_tag = True
+
+from glances import __apiversion__, __version__, psutil_version
 from glances.config import Config
-from glances.globals import WINDOWS
-from glances.processes import sort_processes_key_list
-from glances.logger import logger, LOG_FILENAME
+from glances.globals import WINDOWS, disable, enable
+from glances.logger import LOG_FILENAME, logger
+from glances.processes import sort_processes_stats_list
 
 
-class GlancesMain(object):
+class GlancesMain:
     """Main class to manage Glances instance."""
 
     # Default stats' minimum refresh time is 2 seconds
@@ -70,9 +77,12 @@ Examples of use:
   Connect Glances to a Glances server and export stats to a StatsD server (client mode):
     $ glances -c <ip_server> --export statsd
 
-  Start the client browser (browser mode):
+  Start TUI Central Glances Browser:
     $ glances --browser
 
+  Start WebUI Central Glances Browser:
+    $ glances --browser -w
+
   Display stats to stdout (one stat per line, possible to go inside stats using plugin.attribute):
     $ glances --stdout now,cpu.user,mem.used,load
 
@@ -82,34 +92,111 @@ Examples of use:
   Display CSV stats to stdout (all stats in one line):
     $ glances --stdout-csv now,cpu.user,mem.used,load
 
-  Enable some plugins disabled by default (comma separated list):
+  Enable some plugins disabled by default (comma-separated list):
     $ glances --enable-plugin sensors
 
-  Disable some plugins (comma separated list):
+  Disable some plugins (comma-separated list):
     $ glances --disable-plugin network,ports
 
-  Disable all plugins except some (comma separated list):
+  Disable all plugins except some (comma-separated list):
     $ glances --disable-plugin all --enable-plugin cpu,mem,load
 
 """
 
     def __init__(self):
         """Manage the command line arguments."""
-        # Read the command line arguments
+        self.init_glances()
+
+    def init_glances(self):
+        """Main method to init Glances."""
+        # Read the command line arguments or parse the one given in parameter (parser)
         self.args = self.parse_args()
 
+        # Load the configuration file, if it exists
+        # This function should be called after the parse_args
+        # because the configuration file path can be defined
+        self.config = Config(self.args.conf_file)
+
+        # Init Glances debug mode
+        self.init_debug(self.args)
+
+        # Plugins Glances refresh rate
+        self.init_refresh_rate(self.args)
+
+        # Manage Plugins disable/enable option
+        self.init_plugins(self.args)
+
+        # Init Glances client/server mode
+        self.init_client_server(self.args)
+
+        # Init UI mode
+        self.init_ui_mode(self.args)
+
+        # Init the generate_graph tag
+        # Should be set to True to generate graphs
+        self.args.generate_graph = False
+
+        # Export is only available in standalone or client mode (issue #614)
+        export_tag = self.args.export is not None and any(self.args.export)
+        if WINDOWS and export_tag:
+            # On Windows, export is possible but only in quiet mode
+            # See issue #1038
+            logger.info("On Windows OS, export disable the Web interface")
+            self.args.quiet = True
+            self.args.webserver = False
+        elif not (self.is_standalone() or self.is_client()) and export_tag:
+            logger.critical("Export is only available in standalone or client mode")
+            sys.exit(2)
+
+        # Filter is only available in standalone mode
+        if not self.args.process_filter and not self.is_standalone():
+            logger.debug("Process filter is only available in standalone mode")
+
+        # Focus filter is only available in standalone mode
+        if not self.args.process_focus and not self.is_standalone():
+            logger.debug("Process focus is only available in standalone mode")
+
+        # Cursor option is only available in standalone mode
+        if not self.args.disable_cursor and not self.is_standalone():
+            logger.debug("Cursor is only available in standalone mode")
+
+        # Let the plugins known the Glances mode
+        self.args.is_standalone = self.is_standalone()
+        self.args.is_client = self.is_client()
+        self.args.is_client_browser = self.is_client_browser()
+        self.args.is_server = self.is_server()
+        self.args.is_webserver = self.is_webserver()
+
+        # Check mode compatibility
+        self.check_mode_compatibility()
+
+    def version_msg(self):
+        """Return the version message."""
+        version = f'Glances version:\t{__version__}\n'
+        version += f'Glances API version:\t{__apiversion__}\n'
+        version += f'PsUtil version:\t\t{psutil_version}\n'
+        version += f'Log file:\t\t{LOG_FILENAME}\n'
+        return version
+
     def init_args(self):
         """Init all the command line arguments."""
-        version = 'Glances v{} with PsUtil v{}\nLog file: {}'.format(__version__, psutil_version, LOG_FILENAME)
         parser = argparse.ArgumentParser(
             prog='glances',
             conflict_handler='resolve',
             formatter_class=argparse.RawDescriptionHelpFormatter,
             epilog=self.example_of_use,
         )
-        parser.add_argument('-V', '--version', action='version', version=version)
+        if shtab_tag:
+            shtab.add_argument_to(parser, ["--print-completion"])
+        parser.add_argument('-V', '--version', action='version', version=self.version_msg())
         parser.add_argument('-d', '--debug', action='store_true', default=False, dest='debug', help='enable debug mode')
-        parser.add_argument('-C', '--config', dest='conf_file', help='path to the configuration file')
+        if shtab_tag:
+            parser.add_argument(
+                '-C', '--config', dest='conf_file', help='path to the configuration file'
+            ).complete = shtab.FILE
+        else:
+            parser.add_argument('-C', '--config', dest='conf_file', help='path to the configuration file')
+        parser.add_argument('-P', '--plugins', dest='plugin_dir', help='path to additional plugin directory')
         # Disable plugin
         parser.add_argument(
             '--modules-list',
@@ -122,11 +209,17 @@ Examples of use:
         parser.add_argument(
             '--disable-plugin',
             '--disable-plugins',
+            '--disable',
             dest='disable_plugin',
-            help='disable plugin (comma separated list or all). If all is used, then you need to configure --enable-plugin.',
+            help='disable plugin (comma-separated list or all). If all is used, \
+                then you need to configure --enable-plugin.',
         )
         parser.add_argument(
-            '--enable-plugin', '--enable-plugins', dest='enable_plugin', help='enable plugin (comma separated list)'
+            '--enable-plugin',
+            '--enable-plugins',
+            '--enable',
+            dest='enable_plugin',
+            help='enable plugin (comma-separated list)',
         )
         parser.add_argument(
             '--disable-process',
@@ -149,7 +242,7 @@ Examples of use:
             action='store_true',
             default=False,
             dest='enable_light',
-            help='light mode for Curses UI (disable all but top menu)',
+            help='light mode for Curses UI (disable all but the top menu)',
         )
         parser.add_argument(
             '-0',
@@ -160,7 +253,13 @@ Examples of use:
             help='task\'s cpu usage will be divided by the total number of CPUs',
         )
         parser.add_argument(
-            '-1', '--percpu', action='store_true', default=False, dest='percpu', help='start Glances in per CPU mode'
+            '-1',
+            '--percpu',
+            '--per-cpu',
+            action='store_true',
+            default=False,
+            dest='percpu',
+            help='start Glances in per CPU mode',
         )
         parser.add_argument(
             '-2',
@@ -220,7 +319,7 @@ Examples of use:
         )
         parser.add_argument(
             '--enable-irq', action='store_true', default=False, dest='enable_irq', help='enable IRQ module'
-        ),
+        )
         parser.add_argument(
             '--enable-process-extended',
             action='store_true',
@@ -229,19 +328,31 @@ Examples of use:
             help='enable extended stats on top process',
         )
         parser.add_argument(
-            '--separator',
-            '--enable-separator',
+            '--disable-separator',
+            action='store_false',
+            default=True,
+            dest='enable_separator',
+            help='disable separator in the UI (between top and others modules)',
+        )
+        parser.add_argument(
+            '--disable-cursor',
             action='store_true',
             default=False,
-            dest='enable_separator',
-            help='enable separator in the UI',
-        ),
+            dest='disable_cursor',
+            help='disable cursor (process selection) in the UI',
+        )
+        parser.add_argument(
+            '--arrow-keys-sort',  # See issue #3385
+            action='store_true',
+            default=False,
+            help='Use arrow keys to sort the process list instead of the SHIFT+key combinations',
+        )
         # Sort processes list
         parser.add_argument(
             '--sort-processes',
             dest='sort_processes_key',
-            choices=sort_processes_key_list,
-            help='Sort processes by: {}'.format(', '.join(sort_processes_key_list)),
+            choices=sort_processes_stats_list,
+            help='Sort processes by: {}'.format(', '.join(sort_processes_stats_list)),
         )
         # Display processes list by program name and not by thread
         parser.add_argument(
@@ -253,7 +364,7 @@ Examples of use:
             help='Accumulate processes by program',
         )
         # Export modules feature
-        parser.add_argument('--export', dest='export', help='enable export module (comma separated list)')
+        parser.add_argument('--export', dest='export', help='enable export module (comma-separated list)')
         parser.add_argument(
             '--export-csv-file', default='./glances.csv', dest='export_csv_file', help='file path for CSV exporter'
         )
@@ -273,6 +384,13 @@ Examples of use:
             dest='export_graph_path',
             help='Folder for Graph exporter',
         )
+        parser.add_argument(
+            '--export-process-filter',
+            default=None,
+            type=str,
+            dest='export_process_filter',
+            help='set the export process filter (comma-separated list of regular expression)',
+        )
         # Client/Server option
         parser.add_argument(
             '-c', '--client', dest='client', help='connect to a Glances server by IPv4/IPv6 address or hostname'
@@ -285,7 +403,7 @@ Examples of use:
             action='store_true',
             default=False,
             dest='browser',
-            help='start the client browser (list of servers)',
+            help='start TUI Central Glances Browser (use --browser -w to start WebUI Central Glances Browser)',
         )
         parser.add_argument(
             '--disable-autodiscover',
@@ -300,7 +418,7 @@ Examples of use:
             default=None,
             type=int,
             dest='port',
-            help='define the client/server TCP port [default: {}]'.format(self.server_port),
+            help=f'define the client/server TCP port [default: {self.server_port}]',
         )
         parser.add_argument(
             '-B',
@@ -309,12 +427,13 @@ Examples of use:
             dest='bind_address',
             help='bind server to the given IPv4/IPv6 address or hostname',
         )
+        parser.add_argument('-u', dest='username_used', help='use or define the given username')
         parser.add_argument(
             '--username',
             action='store_true',
             default=False,
             dest='username_prompt',
-            help='define a client/server username',
+            help='define or use an username',
         )
         parser.add_argument(
             '--password',
@@ -323,7 +442,6 @@ Examples of use:
             dest='password_prompt',
             help='define a client/server password',
         )
-        parser.add_argument('-u', dest='username_used', help='use the given client/server username')
         parser.add_argument('--snmp-community', default='public', dest='snmp_community', help='SNMP community')
         parser.add_argument('--snmp-port', default=161, type=int, dest='snmp_port', help='SNMP port')
         parser.add_argument('--snmp-version', default='2c', dest='snmp_version', help='SNMP version (1, 2c or 3)')
@@ -340,7 +458,7 @@ Examples of use:
             default=self.DEFAULT_REFRESH_TIME,
             type=float,
             dest='time',
-            help='set minimum refresh rate in seconds [default: {} sec]'.format(self.DEFAULT_REFRESH_TIME),
+            help=f'set minimum refresh rate in seconds [default: {self.DEFAULT_REFRESH_TIME} sec]',
         )
         parser.add_argument(
             '-w',
@@ -348,14 +466,14 @@ Examples of use:
             action='store_true',
             default=False,
             dest='webserver',
-            help='run Glances in web server mode (bottle needed)',
+            help='run Glances in web server mode (FastAPI, Uvicorn, Jinja2 libs needed)',
         )
         parser.add_argument(
             '--cached-time',
             default=self.cached_time,
             type=int,
             dest='cached_time',
-            help='set the server cache time [default: {} sec]'.format(self.cached_time),
+            help=f'set the server cache time [default: {self.cached_time} sec]',
         )
         parser.add_argument(
             '--stop-after',
@@ -388,6 +506,14 @@ Examples of use:
             dest='process_filter',
             help='set the process filter pattern (regular expression)',
         )
+        # Process will focus on some process (comma-separated list of Glances filter)
+        parser.add_argument(
+            '--process-focus',
+            default=None,
+            type=str,
+            dest='process_focus',
+            help='set a process list to focus on (comma-separated list of Glances filter)',
+        )
         parser.add_argument(
             '--process-short-name',
             action='store_true',
@@ -406,19 +532,19 @@ Examples of use:
             '--stdout',
             default=None,
             dest='stdout',
-            help='display stats to stdout, one stat per line (comma separated list of plugins/plugins.attribute)',
+            help='display stats to stdout, one stat per line (comma-separated list of plugins/plugins.attribute)',
         )
         parser.add_argument(
             '--stdout-json',
             default=None,
             dest='stdout_json',
-            help='display stats to stdout, JSON format (comma separated list of plugins/plugins.attribute)',
+            help='display stats to stdout, JSON format (comma-separated list of plugins/plugins.attribute)',
         )
         parser.add_argument(
             '--stdout-csv',
             default=None,
             dest='stdout_csv',
-            help='display stats to stdout, CSV format (comma separated list of plugins/plugins.attribute)',
+            help='display stats to stdout, CSV format (comma-separated list of plugins/plugins.attribute)',
         )
         parser.add_argument(
             '--issue',
@@ -442,7 +568,18 @@ Examples of use:
             help='test memory leak (python 3.4 or higher needed)',
         )
         parser.add_argument(
-            '--api-doc', default=None, action='store_true', dest='stdout_apidoc', help='display fields descriptions'
+            '--api-doc',
+            default=None,
+            action='store_true',
+            dest='stdout_api_doc',
+            help='display Python API documentation',
+        )
+        parser.add_argument(
+            '--api-restful-doc',
+            default=None,
+            action='store_true',
+            dest='stdout_api_restful_doc',
+            help='display Restful API documentation',
         )
         if not WINDOWS:
             parser.add_argument(
@@ -450,7 +587,7 @@ Examples of use:
                 action='store_true',
                 default=False,
                 dest='no_kernel_threads',
-                help='hide kernel threads in process list (not available on Windows)',
+                help='hide kernel threads in the process list (not available on Windows)',
             )
         parser.add_argument(
             '-b',
@@ -458,7 +595,7 @@ Examples of use:
             action='store_true',
             default=False,
             dest='byte',
-            help='display network rate in byte per second',
+            help='display network rate in bytes per second',
         )
         parser.add_argument(
             '--diskio-show-ramfs',
@@ -474,6 +611,13 @@ Examples of use:
             dest='diskio_iops',
             help='show IO per second in the DiskIO plugin',
         )
+        parser.add_argument(
+            '--diskio-latency',
+            action='store_true',
+            default=False,
+            dest='diskio_latency',
+            help='show IO latency in the DiskIO plugin',
+        )
         parser.add_argument(
             '--fahrenheit',
             action='store_true',
@@ -503,11 +647,10 @@ Examples of use:
             help='disable unicode characters in the curses interface',
         )
         parser.add_argument(
-            '--theme-white',
+            '--hide-public-info',
             action='store_true',
             default=False,
-            dest='theme_white',
-            help='optimize display colors for white background',
+            help='hide public information (like public IP)',
         )
         # Globals options
         parser.add_argument(
@@ -523,41 +666,52 @@ Examples of use:
             default='',
             help='strftime format string for displaying current date in standalone mode',
         )
+        # Fetch
+        parser.add_argument(
+            '--fetch',
+            '--stdout-fetch',
+            action='store_true',
+            default=False,
+            dest='stdout_fetch',
+            help='display a (neo)fetch like summary and exit',
+        )
+        parser.add_argument(
+            '--fetch-template',
+            '--stdout-fetch-template',
+            dest='fetch_template',
+            default='',
+            help='overwrite default fetch template file',
+        )
 
         return parser
 
-    def parse_args(self):
-        """Parse command line arguments."""
-        args = self.init_args().parse_args()
-
-        # Load the configuration file, if it exists
-        # This function should be called after the parse_args
-        # because the configuration file path can be defined
-        self.config = Config(args.conf_file)
-
-        # Debug mode
+    def init_debug(self, args):
+        """Init Glances debug mode."""
         if args.debug:
-            from logging import DEBUG
-
             logger.setLevel(DEBUG)
         else:
-            from warnings import simplefilter
-
             simplefilter("ignore")
 
-        # Plugins refresh rate
+    def init_refresh_rate(self, args):
+        """Init Glances refresh rate"""
         if self.config.has_section('global'):
             global_refresh = self.config.get_float_value('global', 'refresh', default=self.DEFAULT_REFRESH_TIME)
+        else:
+            global_refresh = self.DEFAULT_REFRESH_TIME
+
+        # The configuration key can be overwrite from the command line (-t <time>)
         if args.time == self.DEFAULT_REFRESH_TIME:
             args.time = global_refresh
-        logger.debug('Global refresh rate is set to {} seconds'.format(args.time))
 
-        # Plugins disable/enable
+        logger.debug(f'Global refresh rate is set to {args.time} seconds')
+
+    def init_plugins(self, args):
+        """Init Glances plugins"""
         # Allow users to disable plugins from the glances.conf (issue #1378)
         for s in self.config.sections():
             if self.config.has_section(s) and (self.config.get_bool_value(s, 'disable', False)):
                 disable(args, s)
-                logger.debug('{} disabled by the configuration file'.format(s))
+                logger.debug(f'{s} disabled by the configuration file')
         # The configuration key can be overwrite from the command line
         if args and args.disable_plugin and 'all' in args.disable_plugin.split(','):
             if not args.enable_plugin:
@@ -579,6 +733,27 @@ Examples of use:
             for p in args.export.split(','):
                 setattr(args, 'export_' + p, True)
 
+        # By default help is hidden
+        args.help_tag = False
+
+        # Display Rx and Tx, not the sum for the network
+        args.network_sum = False
+        args.network_cumul = False
+
+        # Processlist is updated in processcount
+        if getattr(args, 'disable_processcount', False):
+            logger.warning('Processcount is disable, so processlist (updated by processcount) is also disable')
+            disable(args, 'processlist')
+        elif getattr(args, 'enable_processlist', False) or getattr(args, 'enable_programlist', False):
+            enable(args, 'processcount')
+
+        # Set a default export_process_filter (with all process) when using the stdout mode
+        if getattr(args, 'stdout', True) and args.process_filter is None:
+            setattr(args, 'export_process_filter', '.*')
+
+    def init_client_server(self, args):
+        """Init Glances client/server mode."""
+
         # Client/server Port
         if args.port is None:
             if args.webserver:
@@ -591,25 +766,19 @@ Examples of use:
                 x if x else y for (x, y) in zip(args.client.partition(':')[::2], (args.client, args.port))
             )
 
-        # Autodiscover
+        # Client autodiscover mode
         if args.disable_autodiscover:
             logger.info("Auto discover mode is disabled")
 
-        # In web server mode
-        if args.webserver:
-            args.process_short_name = True
-
         # Server or client login/password
         if args.username_prompt:
             # Every username needs a password
             args.password_prompt = True
             # Prompt username
-            if args.server:
-                args.username = self.__get_username(description='Define the Glances server username: ')
-            elif args.webserver:
-                args.username = self.__get_username(description='Define the Glances webserver username: ')
+            if args.server or args.webserver:
+                args.username = self.__get_username(description='Enter new username: ')
             elif args.client:
-                args.username = self.__get_username(description='Enter the Glances server username: ')
+                args.username = self.__get_username(description='Enter username: ')
         else:
             if args.username_used:
                 # A username has been set using the -u option ?
@@ -620,21 +789,15 @@ Examples of use:
 
         if args.password_prompt or args.username_used:
             # Interactive or file password
-            if args.server:
+            if args.server or args.webserver:
                 args.password = self.__get_password(
-                    description='Define the Glances server password ({} username): '.format(args.username),
-                    confirm=True,
-                    username=args.username,
-                )
-            elif args.webserver:
-                args.password = self.__get_password(
-                    description='Define the Glances webserver password ({} username): '.format(args.username),
+                    description='Enter new password: ',
                     confirm=True,
                     username=args.username,
                 )
             elif args.client:
                 args.password = self.__get_password(
-                    description='Enter the Glances server password ({} username): '.format(args.username),
+                    description='Enter password: ',
                     clear=True,
                     username=args.username,
                 )
@@ -642,24 +805,19 @@ Examples of use:
             # Default is no password
             args.password = self.password
 
-        # By default help is hidden
-        args.help_tag = False
-
-        # Display Rx and Tx, not the sum for the network
-        args.network_sum = False
-        args.network_cumul = False
-
+    def init_ui_mode(self, args):
         # Manage light mode
-        if args.enable_light:
+        if getattr(args, 'enable_light', False):
             logger.info("Light mode is on")
             args.disable_left_sidebar = True
             disable(args, 'process')
             disable(args, 'alert')
             disable(args, 'amps')
-            disable(args, 'docker')
+            disable(args, 'containers')
+            disable(args, 'vms')
 
         # Manage full quicklook option
-        if args.full_quicklook:
+        if getattr(args, 'full_quicklook', False):
             logger.info("Full quicklook mode")
             enable(args, 'quicklook')
             disable(args, 'cpu')
@@ -668,7 +826,7 @@ Examples of use:
             enable(args, 'load')
 
         # Manage disable_top option
-        if args.disable_top:
+        if getattr(args, 'disable_top', False):
             logger.info("Disable top menu")
             disable(args, 'quicklook')
             disable(args, 'cpu')
@@ -676,58 +834,48 @@ Examples of use:
             disable(args, 'memswap')
             disable(args, 'load')
 
-        # Init the generate_graph tag
-        # Should be set to True to generate graphs
-        args.generate_graph = False
+        # Unicode => No separator
+        if args.disable_unicode:
+            args.enable_separator = False
 
-        # Control parameter and exit if it is not OK
-        self.args = args
-
-        # Export is only available in standalone or client mode (issue #614)
-        export_tag = self.args.export is not None and any(self.args.export)
-        if WINDOWS and export_tag:
-            # On Windows, export is possible but only in quiet mode
-            # See issue #1038
-            logger.info("On Windows OS, export disable the Web interface")
-            self.args.quiet = True
-            self.args.webserver = False
-        elif not (self.is_standalone() or self.is_client()) and export_tag:
-            logger.critical("Export is only available in standalone or client mode")
-            sys.exit(2)
-
-        # Filter is only available in standalone mode
-        if args.process_filter is not None and not self.is_standalone():
-            logger.critical("Process filter is only available in standalone mode")
-            sys.exit(2)
-
-        # Disable HDDTemp if sensors are disabled
-        if getattr(self.args, 'disable_sensors', False):
-            disable(self.args, 'hddtemp')
-            logger.debug("Sensors and HDDTemp are disabled")
-
-        if getattr(self.args, 'trace_malloc', True) and not (PY3 or self.is_standalone()):
-            logger.critical("Option --trace-malloc is only available with Python 3 and terminal mode")
-            sys.exit(2)
-
-        if getattr(self.args, 'memory_leak', True) and not (PY3 or self.is_standalone()):
-            logger.critical("Option --memory-leak is only available with Python 3 and terminal mode")
-            sys.exit(2)
-        elif getattr(self.args, 'memory_leak', True) and (PY3 or self.is_standalone()):
+        # Memory leak
+        if getattr(args, 'memory_leak', False):
             logger.info('Memory leak detection enabled')
-            self.args.quiet = True
-            if not self.args.stop_after:
-                self.args.stop_after = 60
-            self.args.time = 1
-            self.args.disable_history = True
+            args.quiet = True
+            if not args.stop_after:
+                args.stop_after = 60
+            args.time = 1
+            args.disable_history = True
 
-        # Let the plugins known the Glances mode
-        self.args.is_standalone = self.is_standalone()
-        self.args.is_client = self.is_client()
-        self.args.is_client_browser = self.is_client_browser()
-        self.args.is_server = self.is_server()
-        self.args.is_webserver = self.is_webserver()
+        # Disable history if history_size is 0
+        if self.config.has_section('global'):
+            if self.config.get_int_value('global', 'history_size', default=1200) == 0:
+                args.disable_history = True
 
-        return args
+        # Display an information message if history is disabled
+        if args.disable_history:
+            logger.info("Stats history is disabled")
+
+    def parse_args(self):
+        """Parse command line arguments."""
+        return self.init_args().parse_args(sys.argv[1:])
+
+    def check_mode_compatibility(self):
+        """Check mode compatibility"""
+        # Server and Web server are not compatible
+        if self.args.is_server and self.args.is_webserver:
+            logger.critical("Server and Web server modes should not be used together")
+            sys.exit(2)
+
+        # Trace malloc option
+        if getattr(self.args, 'trace_malloc', True) and not self.is_standalone():
+            logger.critical("Option --trace-malloc is only available in the terminal mode")
+            sys.exit(2)
+
+        # Memory leak option
+        if getattr(self.args, 'memory_leak', True) and not self.is_standalone():
+            logger.critical("Option --memory-leak is only available in the terminal mode")
+            sys.exit(2)
 
     def is_standalone(self):
         """Return True if Glances is running in standalone mode."""
@@ -735,11 +883,11 @@ Examples of use:
 
     def is_client(self):
         """Return True if Glances is running in client mode."""
-        return (self.args.client or self.args.browser) and not self.args.server
+        return (self.args.client or self.args.browser) and not self.args.server and not self.args.webserver
 
     def is_client_browser(self):
         """Return True if Glances is running in client browser mode."""
-        return self.args.browser and not self.args.server
+        return self.args.browser and not self.args.server and not self.args.webserver
 
     def is_server(self):
         """Return True if Glances is running in server mode."""
@@ -773,5 +921,5 @@ Examples of use:
         """
         from glances.password import GlancesPassword
 
-        password = GlancesPassword(username=username)
+        password = GlancesPassword(username=username, config=self.get_config())
         return password.get_password(description, confirm, clear)
