@@ -10,9 +10,10 @@
 
 from copy import deepcopy
 from functools import partial, reduce
-from typing import Any, Dict, List, Optional, Tuple
+from itertools import chain
+from typing import Any
 
-from glances.globals import iteritems, itervalues, nativestr
+from glances.globals import nativestr
 from glances.logger import logger
 from glances.plugins.containers.engines import ContainersExtension
 from glances.plugins.containers.engines.docker import DockerExtension, disable_plugin_docker
@@ -50,6 +51,14 @@ fields_description = {
         'description': 'Container CPU consumption',
         'unit': 'percent',
     },
+    'memory_inactive_file': {
+        'description': 'Container memory inactive file',
+        'unit': 'byte',
+    },
+    'memory_limit': {
+        'description': 'Container memory limit',
+        'unit': 'byte',
+    },
     'memory_usage': {
         'description': 'Container memory usage',
         'unit': 'byte',
@@ -70,6 +79,9 @@ fields_description = {
         'description': 'Container network TX bitrate',
         'unit': 'bitpersecond',
     },
+    'ports': {
+        'description': 'Container ports',
+    },
     'uptime': {
         'description': 'Container uptime',
     },
@@ -121,7 +133,7 @@ sort_for_human = {
 }
 
 
-class PluginModel(GlancesPluginModel):
+class ContainersPlugin(GlancesPluginModel):
     """Glances Docker plugin.
 
     stats is a dict: {'version': {...}, 'containers': [{}, {}]}
@@ -142,7 +154,7 @@ class PluginModel(GlancesPluginModel):
         # We want to display the stat in the curse interface
         self.display_curse = True
 
-        self.watchers: Dict[str, ContainersExtension] = {}
+        self.watchers: dict[str, ContainersExtension] = {}
 
         # Init the Docker API
         if not disable_plugin_docker:
@@ -155,6 +167,9 @@ class PluginModel(GlancesPluginModel):
         # Sort key
         self.sort_key = None
 
+        # Set the key's list be disabled in order to only display specific attribute in the container list
+        self.disable_stats = self.get_conf_value('disable_stats')
+
         # Force a first update because we need two update to have the first stat
         self.update()
         self.refresh_timer.set(0)
@@ -165,13 +180,13 @@ class PluginModel(GlancesPluginModel):
         Default value: unix:///run/user/1000/podman/podman.sock
         """
         conf_podman_sock = self.get_conf_value('podman_sock')
-        if len(conf_podman_sock) == 0:
+        if not conf_podman_sock:
             return "unix:///run/user/1000/podman/podman.sock"
         return conf_podman_sock[0]
 
     def exit(self) -> None:
         """Overwrite the exit method to close threads."""
-        for watcher in itervalues(self.watchers):
+        for watcher in self.watchers.values():
             watcher.stop()
 
         # Call the father class
@@ -181,7 +196,7 @@ class PluginModel(GlancesPluginModel):
         """Return the key of the list."""
         return 'name'
 
-    def get_export(self) -> List[Dict]:
+    def get_export(self) -> list[dict]:
         """Overwrite the default export method.
 
         - Only exports containers
@@ -208,13 +223,13 @@ class PluginModel(GlancesPluginModel):
         all=True
         """
         all_tag = self.get_conf_value('all')
-        if len(all_tag) == 0:
+        if not all_tag:
             return False
         return all_tag[0].lower() == 'true'
 
     @GlancesPluginModel._check_decorator
     @GlancesPluginModel._log_result_decorator
-    def update(self) -> List[Dict]:
+    def update(self) -> list[dict]:
         """Update Docker and podman stats using the input method."""
         # Connection should be ok
         if not self.watchers:
@@ -223,19 +238,27 @@ class PluginModel(GlancesPluginModel):
         if self.input_method != 'local':
             return self.get_init_value()
 
-        # Update stats
-        stats = []
-        for engine, watcher in iteritems(self.watchers):
+        def is_key_in_container_and_hidden(container):
+            return (key := container.get('key')) in container and self.is_hide(nativestr(container.get(key)))
+
+        def add_engine_into_container(engine, container):
+            return container | {"engine": engine}
+
+        def get_containers_from_updated_watcher(watcher):
             _, containers = watcher.update(all_tag=self._all_tag())
-            containers_filtered = []
-            for container in containers:
-                container["engine"] = engine
-                if 'key' in container and container['key'] in container:
-                    if not self.is_hide(nativestr(container[container['key']])):
-                        containers_filtered.append(container)
-                else:
-                    containers_filtered.append(container)
-            stats.extend(containers_filtered)
+            return containers
+
+        # Update stats
+        stats = list(
+            chain.from_iterable(
+                (
+                    add_engine_into_container(engine, container)
+                    for container in get_containers_from_updated_watcher(watcher)
+                    if not is_key_in_container_and_hidden(container)
+                )
+                for engine, watcher in self.watchers.items()
+            )
+        )
 
         # Sort and update the stats
         # @TODO: Have a look because sort did not work for the moment (need memory stats ?)
@@ -243,7 +266,7 @@ class PluginModel(GlancesPluginModel):
         return self.stats
 
     @staticmethod
-    def memory_usage_no_cache(mem: Dict[str, float]) -> float:
+    def memory_usage_no_cache(mem: dict[str, float]) -> float:
         """Return the 'real' memory usage by removing inactive_file to usage"""
         # Ref: https://github.com/docker/docker-py/issues/3210
         return mem['usage'] - (mem['inactive_file'] if 'inactive_file' in mem else 0)
@@ -264,18 +287,19 @@ class PluginModel(GlancesPluginModel):
             # CPU alert
             if 'cpu' in i and 'total' in i['cpu']:
                 # Looking for specific CPU container threshold in the conf file
-                alert = self.get_alert(i['cpu']['total'], header=i['name'] + '_cpu', action_key=i['name'])
+                alert = self.get_alert(i['cpu']['total'], header='cpu', action_key=i['name'])
                 if alert == 'DEFAULT':
                     # Not found ? Get back to default CPU threshold value
                     alert = self.get_alert(i['cpu']['total'], header='cpu')
-                self.views[i[self.get_key()]]['cpu']['decoration'] = alert
+                if 'cpu' in self.views[i[self.get_key()]]:
+                    self.views[i[self.get_key()]]['cpu']['decoration'] = alert
             # MEM alert
             if 'memory' in i and 'usage' in i['memory']:
                 # Looking for specific MEM container threshold in the conf file
                 alert = self.get_alert(
                     self.memory_usage_no_cache(i['memory']),
                     maximum=i['memory']['limit'],
-                    header=i['name'] + '_mem',
+                    header='mem',
                     action_key=i['name'],
                 )
                 if alert == 'DEFAULT':
@@ -283,7 +307,8 @@ class PluginModel(GlancesPluginModel):
                     alert = self.get_alert(
                         self.memory_usage_no_cache(i['memory']), maximum=i['memory']['limit'], header='mem'
                     )
-                self.views[i[self.get_key()]]['mem']['decoration'] = alert
+                if 'mem' in self.views[i[self.get_key()]]:
+                    self.views[i[self.get_key()]]['mem']['decoration'] = alert
 
         # Display Engine and Pod name ?
         show_pod_name = False
@@ -343,25 +368,38 @@ class PluginModel(GlancesPluginModel):
 
         ret = self.maybe_add_engine_name_or_pod_line(ret)
 
-        msg = ' {:{width}}'.format('Name', width=name_max_width)
-        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'name' else 'DEFAULT'))
+        if 'name' not in self.disable_stats:
+            msg = ' {:{width}}'.format('Name', width=name_max_width)
+            ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'name' else 'DEFAULT'))
 
-        msgs = ['{:>10}'.format('Status'), '{:>10}'.format('Uptime')]
+        msgs = []
+        if 'status' not in self.disable_stats:
+            msgs.append('{:>10}'.format('Status'))
+        if 'uptime' not in self.disable_stats:
+            msgs.append('{:>10}'.format('Uptime'))
         ret = reduce(self.add_msg_to_line, msgs, ret)
 
-        msg = '{:>6}'.format('CPU%')
-        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'cpu_percent' else 'DEFAULT'))
-        msg = '{:>7}'.format('MEM')
-        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'memory_usage' else 'DEFAULT'))
+        if 'cpu' not in self.disable_stats:
+            msg = '{:>6}'.format('CPU%')
+            ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'cpu_percent' else 'DEFAULT'))
 
-        msgs = [
-            '/{:<7}'.format('MAX'),
-            '{:>7}'.format('IOR/s'),
-            ' {:<7}'.format('IOW/s'),
-            '{:>7}'.format('Rx/s'),
-            ' {:<7}'.format('Tx/s'),
-            ' {:8}'.format('Command'),
-        ]
+        msgs = []
+        if 'mem' not in self.disable_stats:
+            msg = '{:>7}'.format('MEM')
+            ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'memory_usage' else 'DEFAULT'))
+            msgs.append('/{:<7}'.format('MAX'))
+
+        if 'diskio' not in self.disable_stats:
+            msgs.extend(['{:>7}'.format('IOR/s'), ' {:<7}'.format('IOW/s')])
+
+        if 'networkio' not in self.disable_stats:
+            msgs.extend(['{:>7}'.format('Rx/s'), ' {:<7}'.format('Tx/s')])
+
+        if 'ports' not in self.disable_stats:
+            msgs.extend('{:16}'.format('Ports'))
+
+        if 'command' not in self.disable_stats:
+            msgs.append(' {:8}'.format('Command'))
 
         return reduce(self.add_msg_to_line, msgs, ret)
 
@@ -458,6 +496,15 @@ class PluginModel(GlancesPluginModel):
 
         return build_with_this_args
 
+    def build_ports(self, ret, container):
+        if container.get('ports', '') != '':
+            msg = '{:16}'.format(container['ports'])
+        else:
+            msg = '{:16}'.format('_')
+        ret.append(self.curse_add_line(msg, splittable=True))
+
+        return ret
+
     def build_cmd_line(self, ret, container):
         if container['command'] is not None:
             msg = ' {}'.format(container['command'])
@@ -467,14 +514,13 @@ class PluginModel(GlancesPluginModel):
 
         return ret
 
-    def msg_curse(self, args=None, max_width: Optional[int] = None) -> List[str]:
+    def msg_curse(self, args=None, max_width: int | None = None) -> list[str]:
         """Return the dict to display in the curse interface."""
         # Init the return message
         init = []
 
         # Only process if stats exist (and non null) and display plugin enable...
-        conditions = [not self.stats, len(self.stats) == 0, self.is_disabled()]
-        if any(conditions):
+        if any([not self.stats, len(self.stats) == 0, self.is_disabled()]):
             return init
 
         # Build the string message
@@ -499,18 +545,19 @@ class PluginModel(GlancesPluginModel):
 
     def build_container_data(self, name_max_width, args):
         def build_with_this_params(ret, container):
-            steps = [
-                self.maybe_add_engine_name_or_pod_name,
-                self.build_container_name(name_max_width),
-                self.build_status_name,
-                self.build_uptime_line,
-                self.build_cpu_line,
-                self.build_memory_line,
-                self.build_io_line,
-                self.build_net_line(args),
-                self.build_cmd_line,
-            ]
-
+            steps = [self.maybe_add_engine_name_or_pod_name]
+            options = {
+                'name': self.build_container_name(name_max_width),
+                'status': self.build_status_name,
+                'uptime': self.build_uptime_line,
+                'cpu': self.build_cpu_line,
+                'mem': self.build_memory_line,
+                'diskio': self.build_io_line,
+                'networkio': self.build_net_line(args),
+                'ports': self.build_ports,
+                'command': self.build_cmd_line,
+            }
+            steps.extend(v for k, v in options.items() if k not in self.disable_stats)
             return reduce(lambda ret, step: step(ret, container), steps, ret)
 
         return build_with_this_params
@@ -520,29 +567,26 @@ class PluginModel(GlancesPluginModel):
         """Analyse the container status.
         One of created, restarting, running, removing, paused, exited, or dead
         """
-        if status == 'running':
+        if status in ('running', 'healthy'):
             return 'OK'
-        if status == 'dead':
+        if status in ('dead', 'unhealthy'):
             return 'ERROR'
-        if status in ['created', 'restarting', 'exited']:
+        if status in ['created', 'exited']:
             return 'WARNING'
+        if status in ['paused', 'restarting']:
+            return 'CAREFUL'
         return 'INFO'
 
 
-def sort_docker_stats(stats: List[Dict[str, Any]]) -> Tuple[str, List[Dict[str, Any]]]:
+def sort_docker_stats(stats: list[dict[str, Any]]) -> tuple[str, list[dict[str, Any]]]:
     # Make VM sort related to process sort
-    if glances_processes.sort_key == 'memory_percent':
-        sort_by = 'memory_usage'
-        sort_by_secondary = 'cpu_percent'
-    elif glances_processes.sort_key == 'name':
-        sort_by = 'name'
-        sort_by_secondary = 'cpu_percent'
-    else:
-        sort_by = 'cpu_percent'
-        sort_by_secondary = 'memory_usage'
+    sort_by, sort_by_secondary = {
+        'memory_percent': ('memory_usage', 'cpu_percent'),
+        'name': ('name', 'cpu_percent'),
+    }.get(glances_processes.sort_key, ('cpu_percent', 'memory_usage'))
 
     # Sort docker stats
-    sort_stats_processes(
+    stats = sort_stats_processes(
         stats,
         sorted_by=sort_by,
         sorted_by_secondary=sort_by_secondary,
@@ -552,3 +596,6 @@ def sort_docker_stats(stats: List[Dict[str, Any]]) -> Tuple[str, List[Dict[str,
 
     # Return the main sort key and the sorted stats
     return sort_by, stats
+
+
+# End of file
